﻿#include "pch.h"
#include "BasicFingerPaintMain.h"
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace BasicFingerPaint;

using namespace D2D1;
using namespace Platform::Collections;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI;

// Loads and initializes application assets when the application is loaded.
BasicFingerPaintMain::BasicFingerPaintMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
    m_fingerPaintRenderer = std::unique_ptr<BasicFingerPaintRenderer>(new BasicFingerPaintRenderer(m_deviceResources));
	m_deviceResources->RegisterDeviceNotify(this);
}

BasicFingerPaintMain::~BasicFingerPaintMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void BasicFingerPaintMain::CreateWindowSizeDependentResources() 
{
}

// Updates the application state once per frame.
void BasicFingerPaintMain::Update() 
{
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool BasicFingerPaintMain::Render() 
{
    m_fingerPaintRenderer->Render();
	return true;
}

// Notifies renderers that device resources need to be released.
void BasicFingerPaintMain::OnDeviceLost()
{
    m_fingerPaintRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void BasicFingerPaintMain::OnDeviceRecreated()
{
    m_fingerPaintRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}

void BasicFingerPaintMain::SetStrokes(IVector<StrokeInformation^>^ strokesIn)
{
    std::vector<StrokeInfo> strokes;

    for (StrokeInformation^ strokeInformation : strokesIn)
    {
        StrokeInfo strokeInfo;

        for (Windows::Foundation::Point point : strokeInformation->Points)
            strokeInfo.Points.push_back(Point2F(point.X, point.Y));

        strokeInfo.Color = RtColorToDxColor(strokeInformation->Color);
        strokeInfo.Width = strokeInformation->Width;
        strokes.push_back(strokeInfo);
    }

    m_fingerPaintRenderer->SetStrokes(strokes);
}

IVector<StrokeInformation^>^ BasicFingerPaintMain::GetStrokes()
{
    std::vector<StrokeInfo> strokesIn = m_fingerPaintRenderer->GetStrokes();
    Vector<StrokeInformation^>^ strokes = ref new Vector<StrokeInformation^>();         // completedStrokes.size());

    for (StrokeInfo strokeInfo : strokesIn)
    {
        StrokeInformation^ strokeInformation = ref new StrokeInformation();

        for (D2D1_POINT_2F point : strokeInfo.Points)
            strokeInformation->Points->Append(Point(point.x, point.y));

        strokeInformation->Width = strokeInfo.Width;

        ColorF color = strokeInfo.Color;
        strokeInformation->Color = DxColorToRtColor(strokeInfo.Color);
        strokes->Append(strokeInformation);
    }
    return strokes;
}

void BasicFingerPaintMain::BeginStroke(unsigned int id, Point point, float width, Color color)
{
    m_fingerPaintRenderer->BeginStroke(id, 
                                       Point2F(point.X, point.Y), 
                                       width,
                                       RtColorToDxColor(color));
}

void BasicFingerPaintMain::ContinueStroke(unsigned int id, Point point)
{
    m_fingerPaintRenderer->ContinueStroke(id, Point2F(point.X, point.Y));
}

void BasicFingerPaintMain::EndStroke(unsigned int id, Point point)
{
    m_fingerPaintRenderer->EndStroke(id, Point2F(point.X, point.Y));
}

void BasicFingerPaintMain::CancelStroke(unsigned int id)
{
    m_fingerPaintRenderer->CancelStroke(id);
}

void BasicFingerPaintMain::ClearAll()
{
    m_fingerPaintRenderer->ClearAll();
}
