﻿#pragma once

#include "DeviceResources.h"
#include "Content\BasicFingerPaintRenderer.h"
#include "StrokeInformation.h"
#include "StrokeInfo.h"

namespace BasicFingerPaint
{
	class BasicFingerPaintMain : public IDeviceNotify
	{
	public:
		BasicFingerPaintMain(const std::shared_ptr<DeviceResources>& deviceResources);
		~BasicFingerPaintMain();
		void CreateWindowSizeDependentResources();
		void Update();
		bool Render();

		// IDeviceNotify
		virtual void OnDeviceLost();
		virtual void OnDeviceRecreated();

        // Methods for getting stroke collections in and out of the renderer
        void SetStrokes(Windows::Foundation::Collections::IVector<StrokeInformation^>^ strokes);
        Windows::Foundation::Collections::IVector<StrokeInformation^>^ GetStrokes();

        // Methods for processing UI events
        void BeginStroke(unsigned int id, Windows::Foundation::Point point, float width, Windows::UI::Color color);
        void ContinueStroke(unsigned int id, Windows::Foundation::Point point);
        void EndStroke(unsigned int id, Windows::Foundation::Point point);
        void CancelStroke(unsigned int id);
        void ClearAll();

	private:
		// Cached pointer to device resources.
		std::shared_ptr<DeviceResources> m_deviceResources;

		// Renderer
        std::unique_ptr<BasicFingerPaintRenderer> m_fingerPaintRenderer;

        // Conversion
        Windows::UI::Color DxColorToRtColor(D2D1::ColorF color)
        {
            return Windows::UI::ColorHelper::FromArgb(byte(255 * color.a),
                                                      byte(255 * color.r),
                                                      byte(255 * color.g),
                                                      byte(255 * color.b));
        }

        D2D1::ColorF RtColorToDxColor(Windows::UI::Color color)
        {
            return D2D1::ColorF(color.R / 255.0f, color.G / 255.0f, color.B / 255.0f, color.A / 255.0f);
        }
	};
}