#pragma once

#include "..\DeviceResources.h"
#include "StrokeInfo.h"

namespace BasicFingerPaint
{
    class BasicFingerPaintRenderer
    {
    public:
        BasicFingerPaintRenderer(const std::shared_ptr<DeviceResources>& deviceResources);
        void CreateDeviceDependentResources();
        void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
        void Render();

        // Methods for getting stroke collections in and out of the renderer
        void SetStrokes(std::vector<StrokeInfo> strokes);
        std::vector<StrokeInfo> GetStrokes();

        // Methods for processing UI events
        void BeginStroke(unsigned int id, D2D1_POINT_2F point, float width, D2D1::ColorF color);
        void ContinueStroke(unsigned int id, D2D1_POINT_2F point);
        void EndStroke(unsigned int id, D2D1_POINT_2F point);
        void CancelStroke(unsigned int id);
        void ClearAll();

    private:
        // Cached pointer to device resources.
        std::shared_ptr<DeviceResources> m_deviceResources;

        std::vector<StrokeInfo> completedStrokes;
        std::map<unsigned int, StrokeInfo> strokesInProgress;

        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_solidColorBrush;
        Microsoft::WRL::ComPtr<ID2D1StrokeStyle> m_strokeStyle;
        Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;

        Microsoft::WRL::ComPtr<ID2D1PathGeometry> CreatePolylinePathGeometry(const std::vector<D2D1_POINT_2F>& points);
        Microsoft::WRL::ComPtr<ID2D1GeometryRealization> CreateGeometryRealization(StrokeInfo& strokeInfo);
    };
}