﻿//
// MainPage.xaml.h
// Declaration of the MainPage class.
//

#pragma once

#include "DirectXPage.g.h"

#include "DeviceResources.h"
#include "BasicFingerPaintMain.h"
#include "StrokeInformation.h"
#include "NamedColor.h"

namespace BasicFingerPaint
{
	public ref class DirectXPage sealed
	{
    private:
        static Platform::String^ currentDrawingFilename;
        static Platform::String^ filenameExtension;

	public:
		DirectXPage();

		void SaveInternalState(Windows::ApplicationModel::SuspendingOperation^ suspendingOperation, 
                               Windows::Foundation::Collections::IPropertySet^ state);
		void LoadInternalState(Windows::Foundation::Collections::IPropertySet^ state);

	private:
		// XAML low-level rendering event handler.
		void OnRendering(Platform::Object^ sender, Platform::Object^ args);

		// Window event handlers.
		void OnVisibilityChanged(Windows::UI::Core::CoreWindow^ sender, Windows::UI::Core::VisibilityChangedEventArgs^ args);

		// Display properties event handlers.
		void OnDpiChanged(Windows::Graphics::Display::DisplayInformation^ sender, Platform::Object^ args);
		void OnOrientationChanged(Windows::Graphics::Display::DisplayInformation^ sender, Platform::Object^ args);
		void OnDisplayContentsInvalidated(Windows::Graphics::Display::DisplayInformation^ sender, Platform::Object^ args);

		// Other event handlers.
		void AppBarButton_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e);
		void OnCompositionScaleChanged(Windows::UI::Xaml::Controls::SwapChainPanel^ sender, Object^ args);
		void OnSwapChainPanelSizeChanged(Platform::Object^ sender, Windows::UI::Xaml::SizeChangedEventArgs^ e);

		// Resource used to keep track of the rendering event registration.
		Windows::Foundation::EventRegistrationToken m_eventToken;

		// Track our independent input on a background worker thread
		Windows::Foundation::IAsyncAction^ m_inputLoopWorker;

		// Independent input handling functions
		void OnPointerPressed(Platform::Object^ sender, Windows::UI::Core::PointerEventArgs^ e);
		void OnPointerMoved(Platform::Object^ sender, Windows::UI::Core::PointerEventArgs^ e);
		void OnPointerReleased(Platform::Object^ sender, Windows::UI::Core::PointerEventArgs^ e);

		// Resources used to render the DirectX content in the XAML page background.
		std::shared_ptr<DeviceResources> m_deviceResources;
		std::unique_ptr<BasicFingerPaintMain> m_main; 
		bool m_windowVisible;

        // Resources and methods for this particular app
        float m_selectedWidth;
        Windows::UI::Color m_selectedColor;
        Concurrency::critical_section m_criticalSection;

        // UI object handling
        void OnClearAllAppBarButtonClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args);
        void OnColorComboBoxSelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ args);
        void OnWidthComboBoxSelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ args);

        // Open file handlers and methos
        void OnOpenFileFlyoutOpening(Platform::Object^ sender, Platform::Object^ e);
        void OnOpenFileListBoxSelectionChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::SelectionChangedEventArgs^ args);
        void OnOpenFileOpenButtonClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args);
        Concurrency::task<Windows::Foundation::Collections::IVector<StrokeInformation^>^> OpenDrawingAsync(Platform::String^ filename);

        // Save file handlers and methods
        void OnSaveAsFlyoutOpening(Platform::Object^ sender, Platform::Object^ args);
        void OnSaveAsTextBoxTextChanged(Platform::Object^ sender, Windows::UI::Xaml::Controls::TextChangedEventArgs^ args);
        void OnSaveAsSaveButtonClick(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ args);
        Concurrency::task<void> SaveDrawingAsync(Windows::Foundation::Collections::IVector<StrokeInformation^>^ strokes, 
                                                 Platform::String^ filename);

        Concurrency::task<Windows::Foundation::Collections::IVectorView<Windows::Storage::StorageFile^>^> GetFilesAsync();
        Concurrency::task<Windows::Foundation::Collections::IVectorView<Platform::String^>^> GetFileNamesAsync();
    };
}
